window.Gleam.initSearch([{"doc":"redraw","title":"redraw","content":"# Redraw\n\nReact opinionated bindings for Gleam. Use React directly from Gleam, with a\nfriendly API that will never go in your path. Redraw tries to stick to React\nconventions, while providing idiomatic Gleam code. Write interoperable code\nbetween React and Gleam code, reuse existing components, and leverage type-safe\ncomponents &amp; immutable data structure. Forget runtime errors, and write React\ncomponents that just works.\n\n&gt; [!TIP]\n&gt;\n&gt; Have you tried [Lustre](https://lustre.build)? Lustre requires almost no\n&gt; knowledge of JavaScript, no complicated runtimes to get like Node.js, few\n&gt; knowledge of HTML/CSS and other web technologies, less code interfaces to\n&gt; write than Redraw, and is supported by the entire Gleam community! Before\n&gt; launching into Redraw, you should take a look at Lustre, it provides a\n&gt; user-friendly, awesome experience right out-of-the-box for every gleamlins!\n&gt;\n&gt; As a bonus, Lustre is isomorphic, i.e. it can be used as well on client and on\n&gt; server with the same codebase!\n&gt;\n&gt; Redraw assumes you have minimal knowledge on frontend development, and will\n&gt; neither try to ease your learning curve nor simplify and hide the frontend\n&gt; technology stack and complexities. In case you&#39;re not sure what you really\n&gt; need, pick Lustre.\n\n## Overview\n\nRedraw is a package that let you use React in a frontend-only Gleam project. By\nleveraging on the entire JS ecosystem, Redraw help you interop with existing\ncurrent React codebases, or allows you to build your custom codebase and\ncherry-picking the existing components you know and love! Redraw tries to keep\neverything at the lowest level possible, turning all the React niceties into\nGleam niceties. Wherever possible, Redraw tries to stick with Lustre API, to\nhelp you create a codebase mixing the two frameworks. For instance, you could\nbuild libraries targeting both Lustre and Redraw, and using the same design\nsystem for all your products!\n\n## Prerequisites\n\nRedraw assumes that you&#39;re a fluent frontend developer and already understand\nhow React works. If you don&#39;t, it&#39;s best to first learn React and the frontend\necosystem, and come back here later. Meanwhile, you could also take a look at\n[Lustre](https://lustre.build) to create your own application. You&#39;ll find some\ngood tutorials on [react.dev](https://react.dev/), more specifically on\n[&quot;Get Started&quot; page](https://react.dev/learn).\n\nRedraw assumes you have [`node.js`](https://nodejs.org/en) or equivalent as well\nas a modern package manager, i.e. `npm`, `yarn`, `pnpm`, or even `bun`. Redraw\nalso assumes you&#39;re using [`Vite`](https://vitejs.dev/) or an equivalent as\nbuild tool, and will not provide any interface to build your application. In the\nrest of that README, `Vite` will be used as example. It&#39;s up to you to use\nanother bundler if you prefer. Redraw sticks with the modern, up-to-date\nfrontend stack.\n\n## Getting started\n\nCreate the project, and add everything needed to make it work. Choose your\nprefered bundler to start. Create a Vite application, and choose to use\nJavaScript and React. Vite should bundle everything for you directly.\n\n```sh\nnpm create vite@latest\n```\n\n```sh\nyarn create vite\n```\n\n```sh\npnpm create vite\n```\n\n```sh\nbun create vite\n```\n\nFrom the follow-on, `yarn` will be used to illustrate the commands, it&#39;s up to\nyou to see how to use your desired package manager. Then, it&#39;s time to setup the\nproject correctly.\n\n```sh\ncd [project-name]\nyarn install\n\n# Install the Vite Gleam plugin. That plugin is required to tell Vite how to\n# read Gleam files.\nyarn add -D vite-gleam\n\n# If you want to build the project on Vercel or Netlify.\n# @chouqueth/gleam provides a local version of the Gleam compiler installed in\n# your node_modules. You can freely skip that step if you don&#39;t need to build\n# your application remotely or if you&#39;re in control of the environment.\nyarn add -D @chouqueth/gleam\n\n# Remove the files needed for `gleam new` to work.\nmv README.md README.md.old\nmv .gitignore .gitignore.old\n\n# Setup the project.\ngleam new .\ngleam add redraw redraw_dom\n```\n\nNow that everyting is setup, you have to add the `vite-gleam` plugin in\n`vite.config.js`. An example of a `vite.config.js` should look like this.\n\n```javascript\nimport { defineConfig } from &quot;vite&quot;\nimport react from &quot;@vitejs/plugin-react&quot;\nimport gleam from &quot;vite-gleam&quot;\n\n// https://vitejs.dev/config/\nexport default defineConfig({\n  plugins: [react(), gleam()],\n})\n```\n\nYou&#39;re good to go!\n\n## Writing Redraw components\n\nWriting Redraw components is the same as writing React component, with one small\ndifference: wrap the component in `use &lt;- redraw.component()`!\n\n```gleam\nimport redraw\nimport redraw/dom/attribute\nimport redraw/dom/html\n\npub fn gleam_is_awesome() {\n  use &lt;- redraw.component__(&quot;GleamIsAwesome&quot;)\n  html.div([attribute.class(&quot;oh-yeah&quot;)], [\n    html.text(&quot;Yeah, for sure&quot;)\n  ])\n}\n```\n\nWhile this could feels strange at first, you&#39;ll get used to it quickly. To call\nthe component, you&#39;ll need to call the function first, _before definining a new\ncomponent_.\n\n```gleam\nimport redraw\nimport redraw/dom/attribute\nimport redraw/dom/client\nimport redraw/dom/html\n\npub fn main() {\n  let root = root()\n  let assert Ok(root) = client.create_root(&quot;root&quot;)\n  client.render(root, redraw.strict_mode([root()]))\n}\n\nfn root() {\n  // Call `gleam_is_awesome` here, before component creation, otherwise a new\n  // component will be created at each refresh.\n  let gleam_is_awesome = gleam_is_awesome()\n  use &lt;- redraw.component__(&quot;Root&quot;)\n  html.div([], [\n    gleam_is_awesome()\n  ])\n}\n\nfn gleam_is_awesome() {\n  use &lt;- redraw.component__(&quot;GleamIsAwesome&quot;)\n  html.div([attribute.class(&quot;oh-yeah&quot;)], [\n    html.text(&quot;Yeah, for sure&quot;)\n  ])\n}\n```\n\nAnd you know everything to create Redraw components!\n\n## What is the kind of components?\n\nComponents can have different kinds, accepting props, children, ref, or other\nthings. To provide a simpler, usable API in Gleam, props components in Gleam can\nbe List, Tuple or CustomTypes. It means you can totally define your component\nlike this:\n\n```gleam\npub type CounterProps {\n  CounterProps(\n    count: Int,\n    set_count: fn(fn(Int) -&gt; Int) -&gt; Nil,\n  )\n}\n\npub fn counter() {\n  use props: CounterProps &lt;- react.component_(&quot;Counter&quot;)\n  html.button(\n    [events.on_click(fn(_) { props.set_count(fn(count) { count + 1 }) })],\n    [html.text(&quot;count is &quot; &lt;&gt; int.to_string(props.count))],\n  )\n}\n```\n\nbut also like this:\n\n```gleam\npub type CounterProps = #(Int, fn(fn(Int) -&gt; Int) -&gt; Nil)\n\npub fn counter() {\n  use #(count, set_count): CounterProps &lt;- react.component_(&quot;Counter&quot;)\n  html.button(\n    [events.on_click(fn(_) { set_count(fn(count) { count + 1 }) })],\n    [html.text(&quot;count is &quot; &lt;&gt; int.to_string(count))],\n  )\n}\n```\n\nDon&#39;t worry about the translation of data from and to React, Redraw handles the\nhard task for you!\n\n### `component`-family functions\n\nTo define components, you should use `component`, `component_` or `component__`.\nThe difference between the three is the signature of the resulting component.\n`component` accepts props and children, `component_` accepts only props, and\n`component__` do not accept anything. See it as a way to create an empty\ncomponent, used with contexts or internal state for instance. You cannot create\na component that accept children but no props. While it can feel boilerplaty at\nfirst, that is a design decision. Most of the time, components that accept\nchildren also accept props, so it&#39;s not worth creating another API and add\noverhead for a function that will almost not be used.\n\n### `forward_ref`-family functions\n\nDefining components sometimes involves to forward a ref to internal component.\nReact uses the mechanism of `forwardRef` to push a ref, from the parent to a\nnested child. Redraw fully implements forwarded ref components! Use\n`forward_ref` or `forward_ref_` to create a component with props, ref and\nchildren, or only with props and children!\n\n## Some reminders on hooks\n\nNever use hooks outside of custom hooks (functions named `use_[something]`) or\nin components! It means you should never use something like `use_effect` or\n`use_state` outside of the body of `component`-related functions. If you break\nthat rule, while it could seem to work, it&#39;s actually breaking the runtime, and\nit can explode at any time. So keep that rule anytime: no hooks outside of\ncustom hooks or component body.\n\n## Type-checking of hooks\n\nYou could see that hooks often use dependencies array, to determine if they have\nto rerun or not. This is totally supported by Redraw, and leverages on Gleam\nabilities! Always pass a tuple of dependencies to hooks. No type-checking are\ndone at this stage, and probably none will be implemented later, exactly like\nit&#39;s done with React currently. Be careful to provide the correct dependencies.\n\n```gleam\nimport gleam/io\nimport redraw\nimport redraw/attribute\nimport redraw/html\n\nfn gleam_is_awesome() {\n  use &lt;- redraw.component__(&quot;GleamIsAwesome&quot;)\n  redraw.use_effect(fn() {\n    io.println(&quot;Hello from component!&quot;)\n  }, #()) // Passing an empty tuple here is like passing [] in JavaScript.\n  html.div([attribute.class(&quot;oh-yeah&quot;)], [\n    html.text(&quot;Yeah, for sure&quot;)\n  ])\n}\n```\n\n## Using external components\n\nReact is greatly used everywhere. It means a lot of components are already\nusable out-of-the-box. Happily, Redraw provides a way to interop directly with\nthem! Use `to_component` and `to_component_` to integrate a foreign function\ndirectly. Define the correct props, and your work is done! Don&#39;t worry about\nsnake_case and camelCase of props name, Redraw take care of the translation for\nyou. Everytime you put an `Option(a)`, Redraw will also translate it to\n`a | null`, because React use the convention to pass `null` everywhere instead\nof optionals.\n\n```gleam\nimport gleam/option.{type Option}\nimport redraw\nimport redraw/dom/html\n\n// This type will be converted to correct JS props.\npub type ExternalComponentProps {\n  ExternalComponentProps(\n    first_field: Bool,             // firstField: bool\n    second_field: Bool,            // secondField: bool\n    optional_field: Option(String) // optionalField: string | null\n  )\n}\n\n@external(javascript, &quot;external_module&quot;, &quot;ExternalComponent&quot;)\nfn external_component_ffi(props: a) -&gt; redraw.Component\n\nfn external_component() -&gt; fn(ExternalComponentProps) -&gt; redraw.Component {\n  redraw.to_component_(&quot;ExternalComponent&quot;, external_component_ffi)\n}\n\npub fn my_other_component() {\n  let external_component = external_component()\n  use &lt;- redraw.component__(&quot;OtherComponent&quot;)\n  html.div([], [\n    external_component(\n      ExternalComponentProps(\n        first_field: True,\n        second_field: False,\n        optional_field: option.None,\n      )\n    )\n  ])\n}\n```\n\n## Miscellaneous\n\nSome questions, answers, and various informations.\n\n### Is there no linter for Redraw?\n\nAt the moment, Redraw leverages on the Gleam compiler and does not offer linter\nsupport for critical parts like hooks dependencies. A future, complementary\nlinter is planned, and should bridge that gap between Gleam and React. While\nGleam compiler provides all useful information about Gleam code, Redraw linter\nwill focus on specific Redraw requirements.\n\n### What is the state of support for React Native, or any other React flavor?\n\nReact is an isolated packages, and renderers can be various. Redraw has\nsuccessfully been used with Raycast. You can also easily add a `redraw_native`\npackage, and provide an interface for native components. Everything can be done\nquickly and easily, because the entire package is written with as few JS as\npossible. Everything should work almost out-of-the box, because React is already\nworking there. You should take inspiration at how is working `redraw/html`, and\nit could work in the exact same way. It&#39;s only a matter of providing the correct\nComponent to `jsx`.\n\n### Contributing\n\nDo you love the package? You can contribute! Feel free to open a PR, or open an\nissue!\n\n### Why wrapping every components in `use props &lt;- component(name)`?\n\nIf you&#39;re used to React, you know a component is no more than a function,\nreturning a `ReactNode`. Something like this:\n\n```javascript\nfunction GleamIsAwesome(props) {\n  return &lt;div className=&quot;oh-yeah&quot;&gt;Yeah, for sure&lt;/div&gt;\n}\n```\n\nActually, because of JSX, React is a bit lying to all of us, and compiles it to:\n\n```javascript\nimport { jsx } from &quot;react/runtime-jsx&quot;\n\nfunction GleamIsAwesome(props) {\n  return jsx(&quot;div&quot;, { className: &quot;oh-yeah&quot;, children: &quot;Yeah, for sure&quot; })\n}\n```\n\nReact injects a call to `jsx` before every JSX render. This allows for the\nruntime to determine if the function should be called once again with the new\nprovided props. It&#39;s also true for functions.\n\n```javascript\n// That code\nfunction ILoveBeam(props) {\n  return &lt;div&gt;BEAM 💜&lt;/div&gt;\n}\n\nfunction GleamIsAwesome(props) {\n  return (\n    &lt;div className=&quot;oh-yeah&quot;&gt;\n      &lt;ILoveBeam /&gt;\n      Yeah, for sure\n    &lt;/div&gt;\n  )\n}\n\n// Turns into\nimport { jsx, jsxs } from &quot;react/runtime-jsx&quot;\n\nfunction ILoveBeam(props) {\n  return jsx(&quot;div&quot;, { children: &quot;BEAM 💜&quot; })\n}\n\nfunction GleamIsAwesome(props) {\n  return jsxs(&quot;div&quot;, {\n    className: &quot;oh-yeah&quot;,\n    children: [jsx(ILoveBeam, {}), &quot;Yeah, for sure&quot;],\n  })\n}\n```\n\nHere, we got a problem: we cannot inject the `jsx` call before `ILoveBeam` in\nGleam. What we could do is write a function that generate the `jsx` call with an\nother function with `use`, but anonymous functions cannot be used with React:\nReact is doing referential equality for Functional Components. Every component\nshould be defined once and for all.\n\nTo inject the `jsx` call properly, it would requires an additional\ncompilation-step. Because we cannot do this, we pass by a generator function.\nThat&#39;s what `use props &lt;- component(name)` is doing. To get the correct result,\nwe go to a _Component creator_, a function that create a component once and for\nall from a `render` function. Everytime we create a component, we use a render\nfunction and turns it into a proper component to be used with React.\n","url":"index.html"},{"doc":"redraw/error","title":"Error","content":"pub type Error {\n  ExistingContext(name: String)\n  UnknownContext(name: String)\n}\n Main error type. Currently only used in conjuction with `Context` related\n functions.\nExistingContext(name: String)\n Error returned from `create_context_`.\n Context with the corresponding `name` already exists.\n\nUnknownContext(name: String)\n Error returned from `get_context`.\n Context with the corresponding `name` does not exists.\n\nSynonyms:\nredraw/error.Error\nredraw/error Error","url":"redraw/error.html#Error"},{"doc":"redraw/error","title":"redraw/error","content":"","url":"redraw/error.html"},{"doc":"redraw","title":"Component","content":"pub type Component\n Default Node in Redraw. Use `component`-family functions to create components.\n Forwarded ref can be constructed using `forward_ref`-family functions, while\n external components can be used with `to_component`-family functions.\n\nSynonyms:\nredraw.Component\nredraw Component","url":"redraw.html#Component"},{"doc":"redraw","title":"Context","content":"pub type Context(a)\n Pass data without props drilling. \\\n [Documentation](https://react.dev/learn/passing-data-deeply-with-context)\n\nSynonyms:\nredraw.Context\nredraw Context","url":"redraw.html#Context"},{"doc":"redraw","title":"Ref","content":"pub type Ref(a)\n A Ref is a mutable data stored in React, persisted across renders.\n They allow to keep track of a DOM node, a component data, or to store a\n mutable variable in the component, outside of every component lifecycle. \\\n [Documentation](https://react.dev/learn/referencing-values-with-refs)\n\nSynonyms:\nredraw.Ref\nredraw Ref","url":"redraw.html#Ref"},{"doc":"redraw","title":"Suspense","content":"pub type Suspense {\n  Suspense(fallback: Component)\n}\n\nSuspense(fallback: Component)\n\n\nSynonyms:\nredraw.Suspense\nredraw Suspense","url":"redraw.html#Suspense"},{"doc":"redraw","title":"act","content":"pub fn act(act_fn: fn() -&gt; Promise(Nil)) -&gt; Promise(Nil)\n Test helper to apply pending React updates before making assertions. \\\n [Documentation](https://react.dev/reference/react/act)\nSynonyms:\nredraw.act\nredraw act","url":"redraw.html#act"},{"doc":"redraw","title":"component","content":"pub fn component(\n  name name: String,\n  render render: fn(a, List(Component)) -&gt; Component,\n) -&gt; fn(a, List(Component)) -&gt; Component\n Create a Redraw component, with a `name`, and a `render` function. `render`\n will accept props, and a list of children.\nSynonyms:\nredraw.component\nredraw component","url":"redraw.html#component"},{"doc":"redraw","title":"component_","content":"pub fn component_(\n  name name: String,\n  render render: fn(a) -&gt; Component,\n) -&gt; fn(a) -&gt; Component\n Create a Redraw component, with a `name`, and a `render` function. This\n component does not accept children.\nSynonyms:\nredraw.component_\nredraw component_","url":"redraw.html#component_"},{"doc":"redraw","title":"component__","content":"pub fn component__(\n  name name: String,\n  render render: fn() -&gt; Component,\n) -&gt; fn() -&gt; Component\n Create a Redraw component, with a `name` and a `render` function. This\n component does not accept children nor props.\nSynonyms:\nredraw.component__\nredraw component__","url":"redraw.html#component__"},{"doc":"redraw","title":"context","content":"pub fn context(\n  name: String,\n  default_value: fn() -&gt; a,\n) -&gt; Context(a)\n `context` emulates classic Context usage in React. Instead of calling\n `create_context_` and `get_context`, it&#39;s possible to simply call `context`,\n which will get or create the context directly, and allows to write code as\n if Context is globally available. `context` also tries to preserve\n type-checking at most. `context.default_value` is lazily evaluated, meaning\n no additional computations will ever be run.\n\n \n import redraw\n\n const context_name = &quot;MyContextName&quot;\n\n pub type MyContext {\n   MyContext(count: Int, set_count: fn (Int) -&gt; Nil)\n }\n\n fn default_value() {\n   let count = 0\n   les set_count = fn (_) { Nil }\n   MyContext(count:)\n }\n\n pub fn provider() {\n   use _, children &lt;- redraw.component()\n   let context = redraw.context(context_name, default_value)\n   let #(count, set_count) = redraw.use_state(0)\n   redraw.provider(context, MyContext(count:, set_count:), children)\n }\n\n pub fn use_my_context() {\n   let context = redraw.context(context_name, default_value)\n   redraw.use_context(context)\n }\n \n\n `context` should never fail, but it can be wrong if you use an already used\n name.\nSynonyms:\nredraw.context\nredraw context","url":"redraw.html#context"},{"doc":"redraw","title":"create_context","content":"pub fn create_context(\n  default_value default_value: Option(a),\n) -&gt; Context(a)\n Let you create a [context](https://react.dev/learn/passing-data-deeply-with-context) that components can provide or read. \\\n [Documentation](https://react.dev/reference/react/createContext)\nSynonyms:\nredraw.create_context\nredraw create_context","url":"redraw.html#create_context"},{"doc":"redraw","title":"create_context_","content":"pub fn create_context_(\n  name: String,\n  default_value: a,\n) -&gt; Result(Context(a), Error)\n Create a [context](https://react.dev/learn/passing-data-deeply-with-context)\n that components can provide or read.\n Each context is referenced by its name, a little bit like actors in OTP\n (if you&#39;re familiar with Erlang). Because Gleam cannot execute code outside of\n `main` function, creating a context should do some side-effect at startup.\n\n In traditional React code, Context usage is usually written like this.\n\n javascript\n import * as react from &#39;react&#39;\n\n // Create your Context in a side-effectful way.\n const MyContext = react.createContext(defaultValue)\n\n // Create your own provider, wrapping your context.\n export function MyProvider(props) {\n   return &lt;MyContext.Provider&gt;{props.children}&lt;/MyContext.Provider&gt;\n }\n\n // Create your own hook, to simplify usage of your context.\n export function useMyContext() {\n   return react.useContext(MyContext)\n }\n \n\n To simplify and mimic that usage, Redraw wraps Context creation with some\n caching, to emulate a similar behaviour.\n\n \n import redraw\n\n const context_name = &quot;MyContextName&quot;\n\n pub fn my_provider(children) {\n   let assert Ok(context) = redraw.create_context_(context_name, default_value)\n   redraw.provider(context, value, children)\n }\n\n pub fn use_my_context() {\n   let assert Ok(context) = redraw.get_context(context_name)\n   redraw.use_context(context)\n }\n \n\n Be careful, `create_context_` fails if the Context is already defined.\n Choose a full qualified name, hard to overlap with inattention. If\n you want to get a Context in an idempotent way, take a look at [`context()`](#context).\n\n [Documentation](https://react.dev/reference/react/createContext)\nSynonyms:\nredraw.create_context_\nredraw create_context_","url":"redraw.html#create_context_"},{"doc":"redraw","title":"forward_ref","content":"pub fn forward_ref(\n  name name: String,\n  render render: fn(a, Ref(b), List(Component)) -&gt; Component,\n) -&gt; fn(a, Ref(b), List(Component)) -&gt; Component\n Create a Redraw component with children with forwarded ref. \\\n [Documentation](https://fr.react.dev/reference/react/forwardRef)\nSynonyms:\nredraw.forward_ref\nredraw forward_ref","url":"redraw.html#forward_ref"},{"doc":"redraw","title":"forward_ref_","content":"pub fn forward_ref_(\n  name name: String,\n  render render: fn(a, Ref(b)) -&gt; Component,\n) -&gt; fn(a, Ref(b)) -&gt; Component\n Create a Redraw component without children with forwarded ref. \\\n [Documentation](https://react.dev/reference/react/forwardRef)\nSynonyms:\nredraw.forward_ref_\nredraw forward_ref_","url":"redraw.html#forward_ref_"},{"doc":"redraw","title":"fragment","content":"pub fn fragment(children: List(Component)) -&gt; Component\n Fragment allow to group children, without creating a node in the DOM. \\\n [Documentation](https://react.dev/reference/react/Fragment)\nSynonyms:\nredraw.fragment\nredraw fragment","url":"redraw.html#fragment"},{"doc":"redraw","title":"get_context","content":"pub fn get_context(name: String) -&gt; Result(Context(a), Error)\n Get a context. Because of FFI, `get_context` breaks the type-checker. It\n should be considered as unsafe code. As a library author, never exposes\n your context and expect users will call `get_context` themselves, but rather\n exposes a `use_my_context()` function, handling the type-checking for the\n user.\n\n \n import redraw\n\n pub type MyContext {\n   MyContext(value: Int)\n }\n\n /// `use_context` returns `Context(a)`, should it can be safely returned as\n /// `Context(MyContext)`.\n pub fn use_my_context() -&gt; redraw.Context(MyContext) {\n   let context = case redraw.get_context(&quot;MyContextName&quot;) {\n     // Context has been found in the context cache, use it as desired.\n     Ok(context) -&gt; context\n     // Context has not been found. It means the user did not initialised it.\n     Error(_) -&gt; panic as &quot;Unitialised context.&quot;\n   }\n   redraw.use_context(context)\n }\n \nSynonyms:\nredraw.get_context\nredraw get_context","url":"redraw.html#get_context"},{"doc":"redraw","title":"get_current","content":"pub fn get_current(from ref: Ref(a)) -&gt; a\n Get the current value of a ref.\nSynonyms:\nredraw.get_current\nredraw get_current","url":"redraw.html#get_current"},{"doc":"redraw","title":"keyed","content":"pub fn keyed(\n  element: fn(List(Component)) -&gt; Component,\n  content: List(#(String, Component)),\n) -&gt; Component\n Redraw does not support passing key element to components in an easy way like\n React does. To simplify this, it uses the same API than [Lustre](lustre.build)\n to put keys on children.\n \n fn my_component(props, children) {\n   redraw.keyed(my_other_component(props, _), {\n     use item &lt;- list.map(children)\n     #(&quot;my-key&quot;, item)\n   })\n }\n \nSynonyms:\nredraw.keyed\nredraw keyed","url":"redraw.html#keyed"},{"doc":"redraw","title":"memo","content":"pub fn memo(\n  component: fn(a, List(Component)) -&gt; Component,\n) -&gt; fn(a, List(Component)) -&gt; Component\n Memoizes a Redraw component with children. \\\n [Documentation](https://react.dev/reference/react/memo)\nSynonyms:\nredraw.memo\nredraw memo","url":"redraw.html#memo"},{"doc":"redraw","title":"memo_","content":"pub fn memo_(component: fn(a) -&gt; Component) -&gt; fn(a) -&gt; Component\n Memoizes a Redraw component without children. \\\n [Documentation](https://react.dev/reference/react/memo)\nSynonyms:\nredraw.memo_\nredraw memo_","url":"redraw.html#memo_"},{"doc":"redraw","title":"profiler","content":"pub fn profiler(children: List(Component)) -&gt; Component\n Profile allows to measure code performance for a component tree. \\\n [Documentation](https://react.dev/reference/react/Profiler)\nSynonyms:\nredraw.profiler\nredraw profiler","url":"redraw.html#profiler"},{"doc":"redraw","title":"provider","content":"pub fn provider(\n  context context: Context(a),\n  value value: a,\n  children children: List(Component),\n) -&gt; Component\n Wrap your components into a context provider to specify the value of this context for all components inside. \\\n [Documentation](https://react.dev/reference/react/createContext#provider)\nSynonyms:\nredraw.provider\nredraw provider","url":"redraw.html#provider"},{"doc":"redraw","title":"set_current","content":"pub fn set_current(of ref: Ref(a), with value: a) -&gt; Nil\n Set the current value of a ref, overriding its existing content.\nSynonyms:\nredraw.set_current\nredraw set_current","url":"redraw.html#set_current"},{"doc":"redraw","title":"start_transition","content":"pub fn start_transition(scope scope: fn() -&gt; Nil) -&gt; Nil\n Let you update the state without blocking the UI. \\\n [Documentation](https://react.dev/reference/react/startTransition)\nSynonyms:\nredraw.start_transition\nredraw start_transition","url":"redraw.html#start_transition"},{"doc":"redraw","title":"strict_mode","content":"pub fn strict_mode(children: List(Component)) -&gt; Component\n Strict Mode should be enabled during development. \\\n [Documentation](https://react.dev/reference/react/StrictMode)\nSynonyms:\nredraw.strict_mode\nredraw strict_mode","url":"redraw.html#strict_mode"},{"doc":"redraw","title":"suspense","content":"pub fn suspense(\n  props: Suspense,\n  children: List(Component),\n) -&gt; Component\n Suspense allow to display a fallback content while waiting for children to\n finish loading. \\\n [Documentation](https://fr.react.dev/reference/react/Suspense)\nSynonyms:\nredraw.suspense\nredraw suspense","url":"redraw.html#suspense"},{"doc":"redraw","title":"to_component","content":"pub fn to_component(\n  name name: String,\n  component render: fn(a) -&gt; Component,\n) -&gt; fn(a, List(Component)) -&gt; Component\n Convert a React component to a React-redraw component with children. Give it a\n name, and send directly the FFI. Don&#39;t worry about the snake_case over\n camelCase, redraw take care of it for you.\n\n \n import redraw\n\n pub type MyComponentProps {\n   MyComponentProps(\n     first_prop: Bool,\n     second_prop: String,\n   )\n }\n\n @external(javascript, &quot;my_library&quot;, &quot;MyComponent&quot;)\n fn do_my_component(props: MyComponentProps) -&gt; redraw.Component\n\n pub fn my_component() -&gt; fn(MyComponentProps, List(Component)) -&gt; redraw.Component {\n   redraw.to_component(&quot;MyComponent&quot;, do_my_component)\n }\n \nSynonyms:\nredraw.to_component\nredraw to_component","url":"redraw.html#to_component"},{"doc":"redraw","title":"to_component_","content":"pub fn to_component_(\n  name name: String,\n  component render: fn(a) -&gt; Component,\n) -&gt; fn(a) -&gt; Component\n Convert a React Component to a redraw Component without children. Give it a\n name, and send directly the FFI. Don&#39;t worry about the snake_case over\n camelCase, redraw take care of it for you.\n\n \n import redraw\n\n pub type MyComponentProps {\n   MyComponentProps(\n     first_prop: Bool,\n     second_prop: String,\n   )\n }\n\n @external(javascript, &quot;my_library&quot;, &quot;MyComponent&quot;)\n fn do_my_component(props: MyComponentProps) -&gt; redraw.Component\n\n pub fn my_component() -&gt; fn(MyComponentProps) -&gt; redraw.Component {\n   redraw.to_component_(&quot;MyComponent&quot;, do_my_component)\n }\n \nSynonyms:\nredraw.to_component_\nredraw to_component_","url":"redraw.html#to_component_"},{"doc":"redraw","title":"use_callback","content":"pub fn use_callback(fun: a, dependencies: b) -&gt; a\n Let you cache a function definition between re-renders.\n `dependencies` should be a tuple. \\\n [Documentation](https://react.dev/reference/react/useCallback)\nSynonyms:\nredraw.use_callback\nredraw use_callback","url":"redraw.html#use_callback"},{"doc":"redraw","title":"use_context","content":"pub fn use_context(context: Context(a)) -&gt; a\n Let you read and subscribe to [context](https://react.dev/learn/passing-data-deeply-with-context) from your component. \\\n [Documentation](https://react.dev/reference/react/useContext)\nSynonyms:\nredraw.use_context\nredraw use_context","url":"redraw.html#use_context"},{"doc":"redraw","title":"use_debug_value","content":"pub fn use_debug_value(value: a) -&gt; Nil\n Let you add a label to a custom Hook in React DevTools. \\\n [Documentation](https://react.dev/reference/react/useDebugValue)\nSynonyms:\nredraw.use_debug_value\nredraw use_debug_value","url":"redraw.html#use_debug_value"},{"doc":"redraw","title":"use_debug_value_","content":"pub fn use_debug_value_(\n  value: a,\n  formatter: fn(a) -&gt; String,\n) -&gt; Nil\n Let you add a label to a custom Hook in React DevTools, but allow to format\n it before. \\\n [Documentation](https://react.dev/reference/react/useDebugValue)\nSynonyms:\nredraw.use_debug_value_\nredraw use_debug_value_","url":"redraw.html#use_debug_value_"},{"doc":"redraw","title":"use_deferred_value","content":"pub fn use_deferred_value(value: a) -&gt; a\n Let you defer updating a part of the UI. \\\n [Documentation](https://react.dev/reference/react/useDeferredValue)\nSynonyms:\nredraw.use_deferred_value\nredraw use_deferred_value","url":"redraw.html#use_deferred_value"},{"doc":"redraw","title":"use_effect","content":"pub fn use_effect(value: fn() -&gt; Nil, dependencies: a) -&gt; Nil\n Let you synchronize a component with an external system. \\\n [Documentation](https://react.dev/reference/react/useEffect)\nSynonyms:\nredraw.use_effect\nredraw use_effect","url":"redraw.html#use_effect"},{"doc":"redraw","title":"use_effect_","content":"pub fn use_effect_(\n  value: fn() -&gt; fn() -&gt; Nil,\n  dependencies: a,\n) -&gt; Nil\n Let you synchronize a component with an external system. Allow to return\n a cleanup function. \\\n [Documentation](https://react.dev/reference/react/useEffect)\nSynonyms:\nredraw.use_effect_\nredraw use_effect_","url":"redraw.html#use_effect_"},{"doc":"redraw","title":"use_id","content":"pub fn use_id() -&gt; String\n Generate unique IDs that can be passed to accessibility attributes. \\\n [Documentation](https://react.dev/reference/react/useId)\nSynonyms:\nredraw.use_id\nredraw use_id","url":"redraw.html#use_id"},{"doc":"redraw","title":"use_imperative_handle","content":"pub fn use_imperative_handle(\n  ref: Ref(Option(a)),\n  handler: fn() -&gt; a,\n  dependencies: b,\n) -&gt; Nil\n Let you customize the handle exposed as a [ref](https://react.dev/learn/manipulating-the-dom-with-refs).\n Use `use_imperative_handle` when you want to customize the data stored in\n a ref. It&#39;s mostly used in conjuction with `forward_ref`. \\\n [Documentation](https://react.dev/reference/react/useImperativeHandle)\nSynonyms:\nredraw.use_imperative_handle\nredraw use_imperative_handle","url":"redraw.html#use_imperative_handle"},{"doc":"redraw","title":"use_imperative_handle_","content":"pub fn use_imperative_handle_(\n  ref: Ref(a),\n  handler: fn() -&gt; a,\n  dependencies: b,\n) -&gt; Nil\n Let you customize the handle exposed as a [ref](https://react.dev/learn/manipulating-the-dom-with-refs).\n Use `use_imperative_handle` by default, unless you really know what you&#39;re\n doing. \\\n [Documentation](https://react.dev/reference/react/useImperativeHandle)\nSynonyms:\nredraw.use_imperative_handle_\nredraw use_imperative_handle_","url":"redraw.html#use_imperative_handle_"},{"doc":"redraw","title":"use_layout_effect","content":"pub fn use_layout_effect(\n  value: fn() -&gt; Nil,\n  dependencies: a,\n) -&gt; Nil\n Version of useEffect that fires before the browser repaints the screen. \\\n [Documentation](https://react.dev/reference/react/useLayoutEffect)\nSynonyms:\nredraw.use_layout_effect\nredraw use_layout_effect","url":"redraw.html#use_layout_effect"},{"doc":"redraw","title":"use_lazy_state","content":"pub fn use_lazy_state(\n  initial_value: fn() -&gt; a,\n) -&gt; #(a, fn(a) -&gt; Nil)\n Let you add a [state variable](https://react.dev/learn/state-a-components-memory) to your component.\n Allow to create the initial value in a lazy way. \\\n [Documentation](https://react.dev/reference/react/useState)\nSynonyms:\nredraw.use_lazy_state\nredraw use_lazy_state","url":"redraw.html#use_lazy_state"},{"doc":"redraw","title":"use_lazy_state_","content":"pub fn use_lazy_state_(\n  initial_value: fn() -&gt; a,\n) -&gt; #(a, fn(fn(a) -&gt; a) -&gt; Nil)\n Let you add a [state variable](https://react.dev/learn/state-a-components-memory) to your component.\n Allow to create the initial value in a lazy way.\n Give an `updater` function instead of a state setter. \\\n [Documentation](https://react.dev/reference/react/useState)\nSynonyms:\nredraw.use_lazy_state_\nredraw use_lazy_state_","url":"redraw.html#use_lazy_state_"},{"doc":"redraw","title":"use_memo","content":"pub fn use_memo(calculate_value: fn() -&gt; a, dependencies: b) -&gt; a\n Let you cache the result of a calculation between re-renders. \\\n [Documentation](https://react.dev/reference/react/useMemo)\nSynonyms:\nredraw.use_memo\nredraw use_memo","url":"redraw.html#use_memo"},{"doc":"redraw","title":"use_reducer","content":"pub fn use_reducer(\n  reducer: fn(a, b) -&gt; a,\n  initial_state: a,\n) -&gt; #(a, fn(b) -&gt; Nil)\n Let you add a [reducer](https://react.dev/learn/extracting-state-logic-into-a-reducer) to your component. \\\n [Documentation](https://react.dev/reference/react/useReducer)\nSynonyms:\nredraw.use_reducer\nredraw use_reducer","url":"redraw.html#use_reducer"},{"doc":"redraw","title":"use_reducer_","content":"pub fn use_reducer_(\n  reducer: fn(a, b) -&gt; a,\n  initializer: c,\n  init: fn(c) -&gt; a,\n) -&gt; #(a, fn(b) -&gt; Nil)\n Let you add a [reducer](https://react.dev/learn/extracting-state-logic-into-a-reducer) to your component.\n Allow to initialize the store in a custom way. \\\n [Documentation](https://react.dev/reference/react/useReducer)\nSynonyms:\nredraw.use_reducer_\nredraw use_reducer_","url":"redraw.html#use_reducer_"},{"doc":"redraw","title":"use_ref","content":"pub fn use_ref() -&gt; Ref(Option(a))\n Let you reference a value that’s not needed for rendering.\n Most used ref you&#39;ll want to create. They&#39;re automatically created to `None`,\n and can be passed to `ref` prop or `use_imperative_handle`.\n You probably don&#39;t want the ref value to be anything than `Option(a)`, unless\n you have really strong reasons. \\\n [Documentation](https://react.dev/reference/react/useRef)\nSynonyms:\nredraw.use_ref\nredraw use_ref","url":"redraw.html#use_ref"},{"doc":"redraw","title":"use_ref_","content":"pub fn use_ref_(initial_value: a) -&gt; Ref(a)\n Let you reference a value that’s not needed for rendering.\n Use `use_ref` if you&#39;re trying to acquire a reference to a child or to a\n component. Use `use_ref_` when you want to keep track of a data, like if\n you&#39;re doing some side-effects, in conjuction with `get_current` and\n `set_current`. \\\n [Documentation](https://react.dev/reference/react/useRef)\nSynonyms:\nredraw.use_ref_\nredraw use_ref_","url":"redraw.html#use_ref_"},{"doc":"redraw","title":"use_state","content":"pub fn use_state(initial_value: a) -&gt; #(a, fn(a) -&gt; Nil)\n Let you add a [state variable](https://react.dev/learn/state-a-components-memory) to your component. \\\n [Documentation](https://react.dev/reference/react/useState)\nSynonyms:\nredraw.use_state\nredraw use_state","url":"redraw.html#use_state"},{"doc":"redraw","title":"use_state_","content":"pub fn use_state_(\n  initial_value: a,\n) -&gt; #(a, fn(fn(a) -&gt; a) -&gt; Nil)\n Let you add a [state variable](https://react.dev/learn/state-a-components-memory) to your component.\n Give an `updater` function instead of a state setter. \\\n [Documentation](https://react.dev/reference/react/useState)\nSynonyms:\nredraw.use_state_\nredraw use_state_","url":"redraw.html#use_state_"},{"doc":"redraw","title":"use_transition","content":"pub fn use_transition() -&gt; #(Bool, fn() -&gt; Nil)\n Let you update the state without blocking the UI. \\\n [Documentation](https://react.dev/reference/react/useTransition)\nSynonyms:\nredraw.use_transition\nredraw use_transition","url":"redraw.html#use_transition"},{"doc":"redraw","title":"redraw","content":"","url":"redraw.html"},{"doc":"redraw/event","title":"Event","content":"pub type Event\n Synthetic Event sent by React.\n It conforms to the same standard as the underlying DOM events, but\n fixes some browser inconsistencies. \\\n [Documentation](https://react.dev/reference/react-dom/components/common#react-event-object)\n\nSynonyms:\nredraw/event.Event\nredraw/event Event","url":"redraw/event.html#Event"},{"doc":"redraw/event","title":"bubbles","content":"pub fn bubbles(event: Event) -&gt; Bool\n Returns whether the event bubbles through the DOM. \\\n [Documetation](https://developer.mozilla.org/docs/Web/API/Event/bubbles)\nSynonyms:\nredraw/event.bubbles\nredraw/event bubbles","url":"redraw/event.html#bubbles"},{"doc":"redraw/event","title":"cancelable","content":"pub fn cancelable(event: Event) -&gt; Bool\n Returns whether the event can be canceled. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/cancelable)\nSynonyms:\nredraw/event.cancelable\nredraw/event cancelable","url":"redraw/event.html#cancelable"},{"doc":"redraw/event","title":"current_target","content":"pub fn current_target(event: Event) -&gt; Dynamic\n Returns the node to which the current handler is attached in the React tree. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/currentTarget)\nSynonyms:\nredraw/event.current_target\nredraw/event current_target","url":"redraw/event.html#current_target"},{"doc":"redraw/event","title":"default_prevented","content":"pub fn default_prevented(event: Event) -&gt; Bool\n Returns whether `prevent_default` was called. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/defaultPrevented)\nSynonyms:\nredraw/event.default_prevented\nredraw/event default_prevented","url":"redraw/event.html#default_prevented"},{"doc":"redraw/event","title":"event_phase","content":"pub fn event_phase(event: Event) -&gt; Int\n Returns which phase the event is currently in. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/eventPhase)\nSynonyms:\nredraw/event.event_phase\nredraw/event event_phase","url":"redraw/event.html#event_phase"},{"doc":"redraw/event","title":"is_default_prevented","content":"pub fn is_default_prevented(event: Event) -&gt; Bool\n Returns a boolean value indicating whether `prevent_default` was called. \\\n React addition, does not exist in standard browsers.\nSynonyms:\nredraw/event.is_default_prevented\nredraw/event is_default_prevented","url":"redraw/event.html#is_default_prevented"},{"doc":"redraw/event","title":"is_persistent","content":"pub fn is_persistent(event: Event) -&gt; Bool\n Not used with React DOM. With React Native, returns whether persist has been called. \\\n React addition, does not exist in standard browsers.\nSynonyms:\nredraw/event.is_persistent\nredraw/event is_persistent","url":"redraw/event.html#is_persistent"},{"doc":"redraw/event","title":"is_propagation_stopped","content":"pub fn is_propagation_stopped(event: Event) -&gt; Bool\n Returns a boolean value indicating whether `stop_propagation` was called.\n React addition, does not exist in standard browsers.\nSynonyms:\nredraw/event.is_propagation_stopped\nredraw/event is_propagation_stopped","url":"redraw/event.html#is_propagation_stopped"},{"doc":"redraw/event","title":"is_trusted","content":"pub fn is_trusted(event: Event) -&gt; Bool\n Returns whether the event was initiated by user. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/isTrusted)\nSynonyms:\nredraw/event.is_trusted\nredraw/event is_trusted","url":"redraw/event.html#is_trusted"},{"doc":"redraw/event","title":"native_event","content":"pub fn native_event(event: Event) -&gt; Dynamic\n The original browser event object. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event)\nSynonyms:\nredraw/event.native_event\nredraw/event native_event","url":"redraw/event.html#native_event"},{"doc":"redraw/event","title":"persist","content":"pub fn persist(event: Event) -&gt; Event\n Not used with React DOM. With React Native, call this to read event’s properties after the event. \\\n React addition, does not exist in standard browsers.\nSynonyms:\nredraw/event.persist\nredraw/event persist","url":"redraw/event.html#persist"},{"doc":"redraw/event","title":"prevent_default","content":"pub fn prevent_default(event: Event) -&gt; Event\n Prevents the default browser action for the event. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/preventDefault)\nSynonyms:\nredraw/event.prevent_default\nredraw/event prevent_default","url":"redraw/event.html#prevent_default"},{"doc":"redraw/event","title":"stop_propagation","content":"pub fn stop_propagation(event: Event) -&gt; Event\n Stops the event propagation through the React tree. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/stopPropagation)\nSynonyms:\nredraw/event.stop_propagation\nredraw/event stop_propagation","url":"redraw/event.html#stop_propagation"},{"doc":"redraw/event","title":"target","content":"pub fn target(event: Event) -&gt; Dynamic\n Returns the node on which the event has occurred (which could be a distant child). \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/target)\nSynonyms:\nredraw/event.target\nredraw/event target","url":"redraw/event.html#target"},{"doc":"redraw/event","title":"time_stamp","content":"pub fn time_stamp(event: Event) -&gt; Int\n Returns the time when the event occurred. \\\n [Documentation](https://developer.mozilla.org/docs/Web/API/Event/timeStamp)\nSynonyms:\nredraw/event.time_stamp\nredraw/event time_stamp","url":"redraw/event.html#time_stamp"},{"doc":"redraw/event","title":"redraw/event","content":" Event handlers will receive a React event object. It is also sometimes\n known as a “synthetic event”.\n\n ```gleam\n import redraw/dom/events\n import redraw/dom/html\n\n pub fn render() {\n   html.button(\n     [\n       events.on_click(fn (event) {\n         io.debug(event) // React Event\n       })\n     ],\n     [html.text(&quot;Exampe&quot;)]\n   )\n }\n ```\n\n It conforms to the same standard as the underlying DOM events, but fixes\n some browser inconsistencies.\n\n Some React events do not map directly to the browser’s native events.\n For example in `on_mouse_leave`, `event.native_event(event)` will point to a\n `MouseEvent` event.\n The specific mapping is not part of the public API and may change in the\n future. If you need the underlying browser event for some reason, read it\n from `event.native_event(event)`.\n\n Every events have their implementation as `redraw/dom/event/[event_name]`, and\n their corresponding functions are implemented in those. Every time you need\n to use one of the functions defined here, use `to_event` from the\n corresponding module.\n\n [Documentation](https://react.dev/reference/react-dom/components/common#react-event-object)","url":"redraw/event.html"}]);